/*
	Copyright (C) 2006, 2007, 2011 Nicklas Nordborg

	This file is part of the Example Code Package for BASE.
	Available at http://baseplugins.thep.lu.se/
	BASE main site: http://base.thep.lu.se/
	
	This is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.
	
	The software is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.examples.plugins;

import net.sf.basedb.core.BooleanParameterType;
import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.Job;
import net.sf.basedb.core.File;
import net.sf.basedb.core.FileParameterType;
import net.sf.basedb.core.PluginParameter;
import net.sf.basedb.core.ProgressReporter;
import net.sf.basedb.core.RequestInformation;

import net.sf.basedb.core.plugin.About;
import net.sf.basedb.core.plugin.AboutImpl;
import net.sf.basedb.core.plugin.AbstractPlugin;
import net.sf.basedb.core.plugin.AutoDetectingImporter;
import net.sf.basedb.core.plugin.InteractivePlugin;
import net.sf.basedb.core.plugin.GuiContext;
import net.sf.basedb.core.plugin.Plugin;
import net.sf.basedb.core.plugin.Request;
import net.sf.basedb.core.plugin.Response;
import net.sf.basedb.core.signal.SignalHandler;
import net.sf.basedb.core.signal.SignalTarget;
import net.sf.basedb.core.signal.ThreadSignalHandler;

import java.io.InputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Set;


/**
	An example plugin that pretends to import samples. It can't be configured,
	but will ask for a file to import from and if existing samples should be updated 
	or not. It won't actually import any samples, but will report that a few samples
	has been imported.
	
	@base.modified $Date: 2011-09-15 10:32:32 +0200 (Thu, 15 Sep 2011) $
	@author Nicklas
	@version 2.0
*/
public class ExampleImporter 
	extends AbstractPlugin
	implements InteractivePlugin, AutoDetectingImporter, SignalTarget
{

	private static final About about = 
		new AboutImpl
		(
			"Example importer",
			"An example plugin that pretends to import samples. It can't be configured, " +
			"but will ask for a file to import from and if existing samples should be updated " +
			"or not. It won't actually import any samples, but will report that a few samples " +
			"has been imported.",
			"2.17",
			"2006, Base 2 development team",
			null,
			null,
			"http://base.thep.lu.se"
		);

	private static final Set<GuiContext> guiContexts = Collections.singleton(
		new GuiContext(Item.SAMPLE, GuiContext.Type.LIST)
	);

	// Plugin configuration is created by getConfigurePlugin()
	private RequestInformation configurePlugin;

	// Job configuration is created by getConfigureJob()
	private RequestInformation configureJob;
	
	// Parameter that asks for the file to import from
	private PluginParameter<File> fileParameter;
	
	// Parameter that asks if existing items should be updated or not
	private PluginParameter<Boolean> updateExistingParameter;
	
	// 
	private DbControl dc;
	
	// So we can react to ABORT requests
	private ThreadSignalHandler signalHandler;
	
	/**
		Create a new example importer.
	*/
	public ExampleImporter()
	{}
	
	/*
		From the Plugin interface
		-------------------------------------------
	*/
	@Override
	public Plugin.MainType getMainType()
	{
		return Plugin.MainType.IMPORT;
	}
	@Override
	public boolean supportsConfigurations()
	{
		return true;
	}
	@Override
	public boolean requiresConfiguration()
	{
		return false;
	}
	@Override
	public void run(Request request, Response response, ProgressReporter progress)
	{
		// Ensure that the ABORT signal gets sent to this thread
		if (signalHandler != null) signalHandler.setWorkerThread(null);
		
		// Open a connection to the database
		// sc is set by init() method
		DbControl dc = sc.newDbControl();
		try
		{
			// Get the parameters from the job configuration
			boolean updateExisting = (Boolean)job.getValue("updateExisting");
			File f = (File)job.getValue("file");
			
			// Open the file an import it
			f = File.getById(dc, f.getId());
			InputStream in = f.getDownloadStream(0);
			doImport(in, progress);
			in.close();
			
			// Commit the work
			dc.commit();
			response.setDone("Plugin ended successfully");
		}
		catch (Throwable t)
		{
			// All exceptions must be catched and sent back 
			// using the response object
			response.setError(t.getMessage(), Arrays.asList(t));
		}
		finally
		{
			// IMPORTANT!!! Make sure opened connections are closed
			if (dc != null) dc.close();
		}
	}
	// -------------------------------------------
	
	/*
		From the InteractivePlugin interface
		-------------------------------------------
	*/
	/**
		Return a set containing the context [SAMPLE, LIST].
	*/
	@Override
	public Set<GuiContext> getGuiContexts()
	{
		return guiContexts;
	}
	/**
		Always null, we are not dealing with individual items.
	*/
	@Override
	public String isInContext(GuiContext context, Object item)
	{
		return null;
	}
	/**
		The {@link Request#COMMAND_CONFIGURE_PLUGIN} command will not ask
		for any parameters.
		<p>
		The {@link Request#COMMAND_CONFIGURE_JOB} command will ask for
		a file and if existing samples should be updated or ignored.
	*/
	@Override
	public RequestInformation getRequestInformation(GuiContext context, String command)
		throws BaseException
	{
		RequestInformation requestInformation = null;
		if (command.equals(Request.COMMAND_CONFIGURE_PLUGIN))
		{
			requestInformation = getConfigurePlugin();
		}
		else if (command.equals(Request.COMMAND_CONFIGURE_JOB))
		{
			requestInformation = getConfigureJob(context);
		}
		return requestInformation;
	}
	/**
		Store configuration settings for {@link Request#COMMAND_CONFIGURE_PLUGIN} and
		{@link Request#COMMAND_CONFIGURE_JOB}.
	*/
	@Override
	public void configure(GuiContext context, Request request, Response response)
	{
		String command = request.getCommand();
		try
		{
			if (command.equals(Request.COMMAND_CONFIGURE_PLUGIN))
			{
				List<Throwable> errors = 
					validateRequestParameters(getConfigurePlugin().getParameters(), request);
				if (errors != null)
				{
					response.setError(errors.size()+" invalid parameter(s) were found in the request", errors);
					return;
				}

				storeValue(configuration, request, updateExistingParameter);
				response.setDone("Configuration completed");
			}
			else if (command.equals(Request.COMMAND_CONFIGURE_JOB))
			{
				List<Throwable> errors = 
					validateRequestParameters(getConfigureJob(context).getParameters(), request);
				if (errors != null)
				{
					response.setError(errors.size()+" invalid parameter(s) were found in the request", errors);
					return;
				}

				storeValue(job, request, fileParameter);
				storeValue(job, request, updateExistingParameter);
				File importFile = (File)job.getValue(fileParameter.getName());
				response.setSuggestedJobName("Import samples from '" + importFile.getName() + "'");
				response.setDone("Job configuration complete", Job.ExecutionTime.SHORT);
				// TODO - maybe check file size to make a better estimate
			}
		}
		catch (Throwable ex)
		{
			response.setError(ex.getMessage(), Arrays.asList(ex));
		}
	}
	// -------------------------------------------


	/*
		From the AutoDetectingImporter interface
		-------------------------------------------
	*/
	/**
		Always true, since we accept all files.
	*/
	@Override
	public boolean isImportable(InputStream in) 
		throws BaseException
	{
		return true;
	}
	/**
		We don't do anything in this method.
	*/
	@Override
	public void doImport(InputStream in, ProgressReporter progress) 
		throws BaseException
	{
		// Check if the ABORT signal has been sent
		ThreadSignalHandler.checkInterrupted();
		/*
			Pseudo code:
			Parse input stream, for each found sample:
			ThreadSignalHandler.checkInterrupted();  // Do this as the first thing in a loop
			String name = ...
			String externalId = ...
			boolean isNew = false;
			Sample s = getByExternalId(externalId);
			if (s == null)
			{
				s = Sample.getNew(dc);
				s.setName(name) // and other properites
				dc.saveItem(s);
				isNew = true;
			}
			if (updateExisting || isNew)
			{
				s.setName(name);
				s.setExternalId(externalId);
				...
			}
		*/
	}
	// -------------------------------------------
	/*
		From the SignalTarget interface
		-------------------------------------------
	*/
	@Override
	public SignalHandler getSignalHandler()
	{
		signalHandler = new ThreadSignalHandler();
		return signalHandler;
	}
	// -------------------------------------------

	private RequestInformation getConfigureJob(GuiContext context)
	{
		if (configureJob == null)
		{
			// Create file parameter
			fileParameter = new PluginParameter<File>(
				"file",
				"File",
				"The file to import the data from",
				new FileParameterType(null, true, 1)
			);
			
			boolean defaultUpdateExisting = false;
			if (configuration != null)
			{
				defaultUpdateExisting = Boolean.TRUE.equals(configuration.getValue("updateExisting"));
			}
			// Create update existing parameter
			updateExistingParameter = new PluginParameter<Boolean>(
				"updateExisting",
				"Update existing samples",
				"If this option is selected, already existing samples will be updated with the information "+
				"in the file. If this option isn't selected existing samples are left untouched.",
				new BooleanParameterType(defaultUpdateExisting, true)
			);
			
			// Create parameter list and request informaion
			List<PluginParameter<?>> parameters = new ArrayList<PluginParameter<?>>();
			parameters.add(fileParameter);
			parameters.add(updateExistingParameter);
			
			configureJob = new RequestInformation
			(
				Request.COMMAND_CONFIGURE_JOB,
				"Select a file to import samples from",
				"This is an example plugin, it will accept any file but won't import any samples.",
				parameters
			);
		}
		return configureJob;
	}

	private RequestInformation getConfigurePlugin()
	{
		if (configurePlugin == null)
		{
			
			// Create update existing parameter
			updateExistingParameter = new PluginParameter<Boolean>(
				"updateExisting",
				"Update existing samples",
				"If this option is selected, already existing samples will be updated with the information "+
				"in the file. If this option isn't selected existing samples are left untouched.",
				new BooleanParameterType(false, true)
			);
			
			// Create parameter list and request informaion
			List<PluginParameter<?>> parameters = new ArrayList<PluginParameter<?>>();
			parameters.add(updateExistingParameter);
			
			configurePlugin = new RequestInformation
			(
				Request.COMMAND_CONFIGURE_PLUGIN,
				"Default settings",
				"This is an example plugin, it will accept any file but won't import any samples.",
				parameters
			);
		}
		return configurePlugin;
	}

	
}
